
# this is a base template to give you the idea how to work with IDASRVR IPC plugin
# I dont use python much but this will get you started. I used py 3.11 for this.
# full reference implementation is CIDAScript.cls or read the C source

# requires: (thanks mark your a ninja)
#    pip install pypiwin32

import win32con, win32api, win32gui, ctypes, ctypes.wintypes, sys, struct
from array import array
import time
from enum import Enum

WM_DISPLAY_TEXT = 3

#quick call was great for 32bit, 3x faster..actually, if your python is 64bit..
#then quick call (SendMessage) is probably safe for 64 bit number args..
class quickCallMessage(Enum):
    qcmNumFuncs = 13  
    qcmReadByte_32 = 10 
    qcmisX64Disasm = 49
    qcmIDAVer = 50

class COPYDATASTRUCT(ctypes.Structure):
    _fields_ = [
        ('dwData', ctypes.wintypes.LPARAM),
        ('cbData', ctypes.wintypes.DWORD),
        ('lpData', ctypes.c_void_p)
    ]
PCOPYDATASTRUCT = ctypes.POINTER(COPYDATASTRUCT)
      
class Listener:
    # https://stackoverflow.com/questions/5249903/receiving-wm-copydata-in-python   
    def __init__(self):
        self.remote = ''
        self.response = '';
        self.servers = []
        self.HWND_BROADCAST = 0xFFFF
        self.IDASRVR_BROADCAST_MESSAGE = win32gui.RegisterWindowMessage("IDA_SERVER2")
        self.IDA_QUICKCALL_MESSAGE = win32gui.RegisterWindowMessage("IDA_QUICKCALL2")
        
        message_map = {
            win32con.WM_COPYDATA: self.OnCopyData,
            self.IDASRVR_BROADCAST_MESSAGE: self.onBroadCast 
        }

        wc = win32gui.WNDCLASS()
        wc.lpfnWndProc = message_map
        wc.lpszClassName = 'MyWindowClass'
        hinst = wc.hInstance = win32api.GetModuleHandle(None)
        classAtom = win32gui.RegisterClass(wc)
        self.hwnd = win32gui.CreateWindow (
            classAtom,
            "win32gui test",
            0,
            0, 
            0,
            win32con.CW_USEDEFAULT, 
            win32con.CW_USEDEFAULT,
            0, 
            0,
            hinst, 
            None
        )
        #print("Python listening for WM_COPYDATA on hwnd = %d" % self.hwnd)

    def onBroadCast(self, hwnd, msg, wparam, lparam):
        #print("onBroadCast: %x %x %x %x" %  (hwnd, msg, int(wparam), lparam))
        if(lparam != 0): self.servers.append(lparam)
        return 1
    
    def OnCopyData(self, hwnd, msg, wparam, lparam):
        #print("Copy data msg received! hwnd=%d msg=0x%x wparam=0x%x lparam=0x%x" % (hwnd,msg,wparam,lparam))
        pCDS = ctypes.cast(lparam, PCOPYDATASTRUCT)
        if pCDS.contents.dwData != WM_DISPLAY_TEXT:
            #print("Not WM_DISPLAY_TEXT dwData=%d cbData=0x%x lpData=0x%x" % (pCDS.contents.dwData,pCDS.contents.cbData,pCDS.contents.lpData))
            return
        
        #print("WM_DISPLAY_TEXT received cbData=0x%x lpData=0x%x" % (pCDS.contents.cbData,pCDS.contents.lpData))
        msg = ctypes.string_at(pCDS.contents.lpData)
        self.response += msg.decode('utf-8'); #some times it comes in chunked so keep adding
        #print("received: " + self.response)
        return 1
    
    def findIDAWindows(self):
        #print("IDASRVR_BROADCAST_MESSAGE = %x " % self.IDASRVR_BROADCAST_MESSAGE)
        #print("IDA_QUICKCALL_MESSAGE = %x " % self.IDA_QUICKCALL_MESSAGE)
        self.servers = []
        win32gui.SendMessageTimeout(self.HWND_BROADCAST, self.IDASRVR_BROADCAST_MESSAGE, self.hwnd, 0, 0, 1000)

    def sendCommand(self,w, message):
        #print("sending msg'%s' to %s" % (message, w))
        self.response = '';
        isWindow = win32gui.IsWindow(int(w))
        if isWindow != 1:
            print("not a valid hwnd")
            return
        CopyDataStruct = "IIP"
        char_buffer = array('B', message.encode('utf-8'))
        char_buffer_address = char_buffer.buffer_info()[0]
        char_buffer_size = char_buffer.buffer_info()[1]
        cds = struct.pack(CopyDataStruct, WM_DISPLAY_TEXT, char_buffer_size, char_buffer_address)
        v = win32gui.SendMessage(int(w), win32con.WM_COPYDATA, 0, cds)
        #print("done retval = %d" % v)
        return self.response # its syncronous to remote process and back..

    def quickCall(self, w, cmd: quickCallMessage, arg1=0):
        return win32gui.SendMessage(int(w), self.IDA_QUICKCALL_MESSAGE, int(cmd.value), int(arg1))
    
class idaClass:
    def __init__(self):
        self.activeIDA = 0
        self.ipc = Listener()
        print("ipc listening on %x" % self.ipc.hwnd)

    def findIDAWindows(self):
        self.ipc.findIDAWindows()
        return self.ipc.servers
    
    def loadedFile(self):
        return self.ipc.sendCommand(self.activeIDA, "loadedfile:" + str(self.ipc.hwnd))

    def functionStart(self,index):
        tmp = self.ipc.sendCommand(self.activeIDA, "funcstart:"+str(index)+":"+str(self.ipc.hwnd))
        return int(tmp)

    def version(self):
        return self.ipc.quickCall(self.activeIDA, quickCallMessage.qcmIDAVer)
    
    def bits(self):
        return self.ipc.quickCall(self.activeIDA, quickCallMessage.qcmisX64Disasm)

    def numFuncs(self):
        return self.ipc.quickCall(self.activeIDA, quickCallMessage.qcmNumFuncs)

    def xrefsTo(self,offset):
        tmp = self.ipc.sendCommand(self.activeIDA, "refsto:%d:%d" % (offset, self.ipc.hwnd))
        return tmp.replace(",DONE","").split(',') 



ida = idaClass()
servers = ida.findIDAWindows()

if(len(servers)==0):
    print("no open ida windows responded..same priv level? (UIPI sux)")
    exit(0) 

print("found %d open ida windows:" % len(servers))
for index,hwnd in enumerate(servers):
    ida.activeIDA = hwnd
    print("  [%d] hwnd:%x version:%d, bits:%s, funcs:%d, file:%s" % 
             (index, hwnd, ida.version(), ida.bits(), ida.numFuncs(), ida.loadedFile())
    )

ida.activeIDA = servers[0]
func1_va = ida.functionStart(0)
print("function[0].Start: %x" %  func1_va ) 
print("xrefs: " + str(ida.xrefsTo(func1_va)))
print("complete")